//
// FILENAME.
//      LzmaDecode.h
//
//      $PATH:      System\001\Library\CustomizedDecompress\LzmaDecode.h
//
// FUNCTIONAL DESCRIPTION.
//
//
// MODIFICATION HISTORY.
//      $REVISION:  11442               08/14/2010      01:10:01 AM
//      $REPORT:    Cosmetic
//      $DETAILS:   Updated file header to new specification.
//
//      $REVISION:  9781                07/13/2010      03:18:51 AM
//      $REPORT:    Cosmetic CORE043
//      $DETAILS:   Modify data type declaration for coding standard.
//
//      $REVISION:  9737                07/12/2010      06:34:22 PM
//      $REPORT:    Cosmetic CORE043
//      $DETAILS:   Since Foundation cannot be changed without a waiver. Move
//                  CustomizedDecompres to System.
//
// NOTICE.
//      Copyright (C) 2009-2010 Phoenix Technologies Ltd.  All Rights Reserved.
//      LZMA SDK 4.05 Copyright (c) 1999-2004 Igor Pavlov (2004-08-25)
//      http://www.7-zip.org/
//
//      LZMA SDK is licensed under two licenses:
//      1) GNU Lesser General Public License (GNU LGPL)
//      2) Common Public License (CPL)
//      It means that you can select one of these two licenses and
//      follow rules of that license.
//
//      SPECIAL EXCEPTION:
//      Igor Pavlov, as the author of this code, expressly permits you to
//      statically or dynamically link your code (or bind by name) to the
//      interfaces of this file without subjecting your linked code to the
//      terms of the CPL or GNU LGPL. Any modifications or additions
//      to this file, however, are subject to the LGPL or CPL terms.
//

#ifndef __LZMADECODE_H
#define __LZMADECODE_H

#ifdef _LZMA_PROB32
#define CProb UINT32
#else
#define CProb unsigned short
#endif

#define LZMA_RESULT_OK 0
#define LZMA_RESULT_DATA_ERROR 1
#define LZMA_RESULT_NOT_ENOUGH_MEM 2

#ifdef _LZMA_IN_CB
typedef struct _ILzmaInCallback
{
  INT32 (*Read)(void *object, UINT8 **buffer, UINT32 *bufferSize);
} ILzmaInCallback;
#endif

#define LZMA_BASE_SIZE 1846
#define LZMA_LIT_SIZE 768

//
// bufferSize = (LZMA_BASE_SIZE + (LZMA_LIT_SIZE << (lc + lp)))* sizeof(CProb)
//    bufferSize += 100 in case of _LZMA_OUT_READ
//    by default CProb is unsigned short,
//    but if specify _LZMA_PROB_32, CProb will be UINT32
//

#ifdef _LZMA_OUT_READ
INT32 LzmaDecoderInit(
    UINT8 *buffer, UINT32 bufferSize,
    INT32 lc, INT32 lp, INT32 pb,
    UINT8 *dictionary, UINT32 dictionarySize,
  #ifdef _LZMA_IN_CB
    ILzmaInCallback *inCallback
  #else
    UINT8 *inStream, UINT32 inSize
  #endif
);
#endif

INT32 LzmaDecode(
    UINT8 *buffer,
  #ifndef _LZMA_OUT_READ
    UINT32 bufferSize,
    INT32 lc, INT32 lp, INT32 pb,
  #ifdef _LZMA_IN_CB
    ILzmaInCallback *inCallback,
  #else
    UINT8 *inStream, UINT32 inSize,
  #endif
  #endif
    UINT8 *outStream, UINT32 outSize,
    UINT32 *outSizeProcessed);

#endif
